/*
 * Copyright 2006 Integsystem Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package jp.integ.struts.config.rule;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.seasar.framework.util.StringUtil;
import org.seasar.struts.config.rule.CommonNamingRule;

/**
 * パッケージルートの処理ルールクラスです。
 * 
 * @author Jun Futagawa (Integsystem Corporation)
 * @version $Date: 2006-03-10 00:54:16 +0900 $
 */
public class QualifiedRule {
	/** 大文字をパスに変換するためのパターンを表します。 */
	private static final Pattern upperNameToUriPattern = Pattern
			.compile("[0-9a-z_-]+|[A-Z][0-9a-z]+");

	/**
	 * 指定されたパッケージルールからアクションパスを取得します。
	 * 
	 * @param packageRule パッケージルール
	 * @param className アクションクラス名
	 * @param splitUpperName 大文字をパスに変換するかどうか
	 * @return アクションパス
	 */
	public static String getActionPath(String packageRule, String className,
			boolean splitUpperName) {
		String actionPath = getActionPath(packageRule, className);
		if (splitUpperName)
			actionPath = QualifiedRule.splitUpperName(actionPath);
		return actionPath;
	}

	/**
	 * 指定されたパッケージルールからアクションパスを取得します。
	 * 
	 * @param packageRule パッケージルール
	 * @param className アクションクラス名
	 * @return アクションパス
	 */
	public static String getActionPath(String packageRule, String className) {
		String result = "";
		if (!StringUtil.isEmpty(packageRule)) {
			String[] rules = packageRule.split(",");
			for (int i = 0; i < rules.length; i++) {
				String tmp = getActionPathBySingleRule(rules[i], className);
				if (result.equals("") || tmp.length() < result.length()) {
					result = tmp;
				}
			}
		} else {
			result = getActionPathBySingleRule(packageRule, className);
		}
		return result;
	}

	/**
	 * 指定されたパッケージルールからアクションパスを取得します。
	 * 
	 * @param packageRule
	 * @param className
	 * @return アクションパス
	 */
	public static String getActionPathBySingleRule(String packageRule,
			String className) {
		if (StringUtil.isEmpty(className))
			return null;
		if (!StringUtil.isEmpty(packageRule)) {
			packageRule = packageRule.replaceAll("\\(", "(?:").replaceAll(
					"\\)", ".)?");
			packageRule = packageRule.replaceAll("\\.", "\\\\.").replaceAll(
					"\\*", "(.*?)");
			packageRule += "(.*?)$";
			Pattern pattern = Pattern.compile(packageRule);
			Matcher matcher = pattern.matcher(className);
			if (matcher.find()) {
				StringBuffer buffer = new StringBuffer();
				for (int i = 1; i < matcher.groupCount() + 1; i++) {
					buffer
							.append(
									CommonNamingRule.decapitalizeName(matcher
											.group(i))).append(".");
				}
				buffer.deleteCharAt(buffer.length() - 1);
				className = buffer.toString();
			}
		}
		className = "." + CommonNamingRule.decapitalizeName(className);
		className = getLastDecapitalizeName(className);
		className = className.replaceAll("\\.{2}?", "\\.").replaceAll("\\.",
				"/");
		return className.replaceFirst("Action$", "");
	}

	/**
	 * 大文字をパスに変換します。
	 * 
	 * @param uri パス表現
	 * @return 大文字をパスに変換したパス
	 */
	public static String splitUpperName(String uri) {
		if (StringUtil.isEmpty(uri))
			return null;
		int lastIndex = uri.length();
		Matcher matcher = upperNameToUriPattern.matcher(uri);
		StringBuffer buffer = new StringBuffer();
		while (matcher.find()) {
			String group = matcher.group().toLowerCase();
			if (matcher.end() == lastIndex) {
				buffer.append(group);
			} else {
				buffer.append(group).append("/");
			}
		}
		return "/" + buffer.toString();
	}

	/**
	 * 指定された名前の最後の . の次の1文字を小文字にします。
	 * 
	 * @param name
	 * @return
	 */
	private static String getLastDecapitalizeName(String name) {
		int index = name.lastIndexOf(".");
		if (index > 0 && name.length() > index) {
			index += 1;
			char chars[] = name.toCharArray();
			chars[index] = Character.toLowerCase(chars[index]);
			return new String(chars);
		}
		return name;
	}
}
